@@ -93,7 +93,6 @@ tp_get_average_touches_delta(struct tp_dispatch *tp)
 static void
 tp_gesture_start(struct tp_dispatch *tp, uint64_t time)
 {
-	struct libinput *libinput = tp_libinput_context(tp);
 	const struct normalized_coords zero = { 0.0, 0.0 };
 
 	if (tp->gesture.started)
@@ -102,9 +101,9 @@ tp_gesture_start(struct tp_dispatch *tp, uint64_t time)
 	switch (tp->gesture.state) {
 	case GESTURE_STATE_NONE:
 	case GESTURE_STATE_UNKNOWN:
-		log_bug_libinput(libinput,
-				 "%s in unknown gesture mode\n",
-				 __func__);
+		evdev_log_bug_libinput(tp->device,
+				       "%s in unknown gesture mode\n",
+				       __func__);
 		break;
 	case GESTURE_STATE_SCROLL:
 		/* NOP */
@@ -185,20 +184,19 @@ static uint32_t
 tp_gesture_get_direction(struct tp_dispatch *tp, struct tp_touch *touch,
 			 unsigned int nfingers)
 {
-	struct normalized_coords normalized;
+	struct phys_coords mm;
 	struct device_float_coords delta;
-	double move_threshold = TP_MM_TO_DPI_NORMALIZED(1);
+	double move_threshold = 1.0; /* mm */
 
 	move_threshold *= (nfingers - 1);
 
 	delta = device_delta(touch->point, touch->gesture.initial);
+	mm = tp_phys_delta(tp, delta);
 
-	normalized = tp_normalize_delta(tp, delta);
-
-	if (normalized_length(normalized) < move_threshold)
+	if (length_in_mm(mm) < move_threshold)
 		return UNDEFINED_DIRECTION;
 
-	return normalized_get_direction(normalized);
+	return phys_get_direction(mm);
 }
 
 static void
@@ -481,10 +479,10 @@ tp_gesture_post_gesture(struct tp_dispatch *tp, uint64_t time)
 		tp->gesture.state =
 			tp_gesture_handle_state_pinch(tp, time);
 
-	log_debug(tp_libinput_context(tp),
-		  "gesture state: %s → %s\n",
-		  gesture_state_to_str(oldstate),
-		  gesture_state_to_str(tp->gesture.state));
+	evdev_log_debug(tp->device,
+			"gesture state: %s → %s\n",
+			gesture_state_to_str(oldstate),
+			gesture_state_to_str(tp->gesture.state));
 }
 
 void
@@ -531,7 +529,6 @@ tp_gesture_stop_twofinger_scroll(struct tp_dispatch *tp, uint64_t time)
 static void
 tp_gesture_end(struct tp_dispatch *tp, uint64_t time, bool cancelled)
 {
-	struct libinput *libinput = tp_libinput_context(tp);
 	enum tp_gesture_state state = tp->gesture.state;
 
 	tp->gesture.state = GESTURE_STATE_NONE;
@@ -542,9 +539,9 @@ tp_gesture_end(struct tp_dispatch *tp, uint64_t time, bool cancelled)
 	switch (state) {
 	case GESTURE_STATE_NONE:
 	case GESTURE_STATE_UNKNOWN:
-		log_bug_libinput(libinput,
-				 "%s in unknown gesture mode\n",
-				 __func__);
+		evdev_log_bug_libinput(tp->device,
+				       "%s in unknown gesture mode\n",
+				       __func__);
 		break;
 	case GESTURE_STATE_SCROLL:
 		tp_gesture_stop_twofinger_scroll(tp, time);
