@@ -70,6 +70,16 @@ struct pad_dispatch {
 	} modes;
 };
 
+static inline struct pad_dispatch*
+pad_dispatch(struct evdev_dispatch *dispatch)
+{
+	struct pad_dispatch *p;
+
+	evdev_verify_dispatch_type(dispatch, DISPATCH_TABLET_PAD);
+
+	return container_of(dispatch, p, base);
+}
+
 static inline struct libinput *
 pad_libinput_context(const struct pad_dispatch *pad)
 {
